@@ -14,7 +14,7 @@
 // (c) Ulf Frisk, 2020-2021
 // Author: Ulf Frisk, pcileech@frizk.net
 //
-// Header Version: 2.5
+// Header Version: 2.8
 //
 
 #ifndef __LEECHCORE_H__
@@ -28,23 +28,26 @@ extern "C" {
 //-----------------------------------------------------------------------------
 
 #ifdef _WIN32
-#include <Windows.h>
 
+#include <Windows.h>
 #define EXPORTED_FUNCTION                   __declspec(dllexport)
-typedef ULONG64                             QWORD, *PQWORD;
+typedef unsigned __int64                    QWORD, *PQWORD;
+
 #endif /* _WIN32 */
 #ifdef LINUX
+
 #include <inttypes.h>
 #include <stdlib.h>
-
 #define EXPORTED_FUNCTION                   __attribute__((visibility("default")))
-typedef void                                VOID, *PVOID, *HANDLE;
-typedef long long unsigned int              QWORD, *PQWORD, ULONG64;
-typedef uint32_t                            DWORD, *PDWORD, BOOL;
-typedef uint8_t                             BYTE, *PBYTE;
-typedef char                                CHAR, *LPSTR;
-typedef const char                          *LPCSTR;
-typedef wchar_t                             WCHAR, *PWCHAR, *LPWSTR;
+typedef void                                VOID, *PVOID, *HANDLE, **PHANDLE, *HMODULE;
+typedef long long unsigned int              QWORD, *PQWORD, ULONG64, *PULONG64;
+typedef size_t                              SIZE_T, *PSIZE_T;
+typedef uint64_t                            FILETIME, *PFILETIME;
+typedef uint32_t                            DWORD, *PDWORD, *LPDWORD, BOOL, *PBOOL, NTSTATUS;
+typedef uint16_t                            WORD, *PWORD;
+typedef uint8_t                             BYTE, *PBYTE, *LPBYTE, UCHAR;
+typedef char                                CHAR, *PCHAR, *LPSTR, *LPCSTR;
+typedef uint16_t                            WCHAR, *PWCHAR, *LPWSTR, *LPCWSTR;
 #define MAX_PATH                            260
 #define _In_
 #define _In_z_
@@ -52,16 +55,22 @@ typedef wchar_t                             WCHAR, *PWCHAR, *LPWSTR;
 #define _In_reads_(x)
 #define _In_reads_opt_(x)
 #define _Inout_
+#define _Inout_bytecount_(x)
+#define _Inout_opt_
+#define _Inout_updates_opt_(x)
 #define _Out_
 #define _Out_opt_
 #define _Out_writes_(x)
+#define _Out_writes_bytes_opt_(x)
 #define _Out_writes_opt_(x)
-#define _Inout_updates_opt_(x)
+#define _Out_writes_to_(x,y)
+#define _When_(x,y)
 #define _Frees_ptr_opt_
 #define _Post_ptr_invalid_
 #define _Check_return_opt_
 #define _Printf_format_string_
 #define _Success_(x)
+
 #endif /* LINUX */
 
 
@@ -161,7 +170,10 @@ typedef struct tdMEM_SCATTER {
     DWORD version;                          // MEM_SCATTER_VERSION
     BOOL f;                                 // TRUE = success data in pb, FALSE = fail or not yet read.
     QWORD qwA;                              // address of memory to read
-    PBYTE pb;                               // buffer to hold memory contents
+    union {
+        PBYTE pb;                           // buffer to hold memory contents
+        QWORD _Filler;
+    };
     DWORD cb;                               // size of buffer to hold memory contents.
     DWORD iStack;                           // internal stack pointer
     QWORD vStack[MEM_SCATTER_STACK_SIZE];   // internal stack
@@ -388,7 +400,7 @@ EXPORTED_FUNCTION BOOL LcCommand(
 #define LC_OPT_FPGA_DELAY_WRITE                     0x0300000700000000  // RW - uS
 #define LC_OPT_FPGA_DELAY_READ                      0x0300000800000000  // RW - uS
 #define LC_OPT_FPGA_RETRY_ON_ERROR                  0x0300000900000000  // RW
-#define LC_OPT_FPGA_DEVICE_ID                       0x0300008000000000  // R
+#define LC_OPT_FPGA_DEVICE_ID                       0x0300008000000000  // RW - bus:dev:fn (ex: 04:00.0 == 0x0400).
 #define LC_OPT_FPGA_FPGA_ID                         0x0300008100000000  // R
 #define LC_OPT_FPGA_VERSION_MAJOR                   0x0300008200000000  // R
 #define LC_OPT_FPGA_VERSION_MINOR                   0x0300008300000000  // R
@@ -425,6 +437,14 @@ EXPORTED_FUNCTION BOOL LcCommand(
 
 #define LC_CMD_AGENT_EXEC_PYTHON                    0x8000000100000000  // RW - [lo-dword: optional timeout in ms]
 #define LC_CMD_AGENT_EXIT_PROCESS                   0x8000000200000000  //    - [lo-dword: process exit code]
+#define LC_CMD_AGENT_VFS_LIST                       0x8000000300000000  // RW
+#define LC_CMD_AGENT_VFS_READ                       0x8000000400000000  // RW
+#define LC_CMD_AGENT_VFS_WRITE                      0x8000000500000000  // RW
+#define LC_CMD_AGENT_VFS_OPT_GET                    0x8000000600000000  // RW
+#define LC_CMD_AGENT_VFS_OPT_SET                    0x8000000700000000  // RW
+
+#define LC_CMD_AGENT_VFS_REQ_VERSION                0xfeed0001
+#define LC_CMD_AGENT_VFS_RSP_VERSION                0xfeee0001
 
 #define LC_STATISTICS_VERSION                       0xe1a10002
 #define LC_STATISTICS_ID_OPEN                       0x00
@@ -437,6 +457,28 @@ EXPORTED_FUNCTION BOOL LcCommand(
 #define LC_STATISTICS_ID_COMMAND                    0x07
 #define LC_STATISTICS_ID_MAX                        0x07
 
+typedef struct tdLC_CMD_AGENT_VFS_REQ {
+    DWORD dwVersion;
+    DWORD _FutureUse;
+    CHAR uszPathFile[2*MAX_PATH];   // file path to list/read/write
+    union {
+        QWORD qwOffset;             // offset to read/write
+        QWORD fOption;              // option to get/set (qword data in *pb)
+    };
+    DWORD dwLength;                 // length to read
+    DWORD cb;
+    BYTE pb[0];
+} LC_CMD_AGENT_VFS_REQ, *PLC_CMD_AGENT_VFS_REQ;
+
+typedef struct tdLC_CMD_AGENT_VFS_RSP {
+    DWORD dwVersion;
+    DWORD dwStatus;                 // ntstatus of read/write
+    DWORD cbReadWrite;              // number of bytes read/written
+    DWORD _FutureUse[2];
+    DWORD cb;
+    BYTE pb[0];
+} LC_CMD_AGENT_VFS_RSP, *PLC_CMD_AGENT_VFS_RSP;
+
 static LPCSTR LC_STATISTICS_NAME[] = {
     "LcOpen",
     "LcRead",
