@@ -16,6 +16,8 @@
 #define DUMP_VALID_DUMP             0x504d5544
 #define DUMP_VALID_DUMP64           0x34365544
 #define DUMP_TYPE_FULL              1
+#define DUMP_TYPE_BITMAP_FULL       5
+#define DUMP_TYPE_ACTIVE_MEMORY     6
 #define IMAGE_FILE_MACHINE_I386     0x014c
 #define IMAGE_FILE_MACHINE_AMD64    0x8664
 #define _PHYSICAL_MEMORY_MAX_RUNS   0x20
@@ -44,6 +46,15 @@ typedef struct {
     _PHYSICAL_MEMORY_RUN32 Run[_PHYSICAL_MEMORY_MAX_RUNS];
 } _PHYSICAL_MEMORY_DESCRIPTOR32, *_PPHYSICAL_MEMORY_DESCRIPTOR32;
 
+typedef struct tdDUMP_HEADER_BITMAP_FULL64 {
+    QWORD Signature;        // + 0x00
+    QWORD _Filler[3];       // + 0x08
+    QWORD cbFileBase;       // + 0x20
+    QWORD cPages;           // + 0x28
+    QWORD cBits;            // + 0x30
+    BYTE pbBitmap[0];       // + 0x38
+} _DUMP_HEADER_BITMAP_FULL64, *P_DUMP_HEADER_BITMAP_FULL64;
+
 #define CDMP_DWORD(o)                               (*(PDWORD)(ctx->CrashOrCoreDump.pbHdr + o))
 #define CDMP_QWORD(o)                               (*(PQWORD)(ctx->CrashOrCoreDump.pbHdr + o))
 #define VMM_PTR_OFFSET_DUAL(f32, pb, o32, o64)      ((f32) ? *(PDWORD)((o32) + (PBYTE)(pb)) : *(PQWORD)((o64) + (PBYTE)(pb)))
@@ -289,6 +300,80 @@ fail:
     if(pFile) { fclose(pFile); }
 }
 
+/*
+* Parsing of the Full Bitmap Microsoft Crashdump file (currently only 64-bits).
+* -- ctxLC
+* -- return
+*/
+BOOL DeviceFile_MsCrashCoreDumpInitialize_BitmapFullOrActiveMemory(_In_ PLC_CONTEXT ctxLC)
+{
+    PDEVICE_CONTEXT_FILE ctx = (PDEVICE_CONTEXT_FILE)ctxLC->hDevice;
+    _DUMP_HEADER_BITMAP_FULL64 hdr = { 0 };
+    PBYTE pb = NULL;
+    BOOL fResult = FALSE, fPageValid = FALSE;
+    QWORD cb, cbFileBase, iPageBase, iPage, cMaxBits, iPageEx, b;
+    // 1: fetch header:
+    _fseeki64(ctx->pFile, 0x2000, SEEK_SET);
+    fread(&hdr, 1, sizeof(_DUMP_HEADER_BITMAP_FULL64), ctx->pFile);
+    if((hdr.Signature != 0x504d5544504d4446) && (hdr.Signature != 0x504d5544504d4453)) { goto fail; }   // !'FDMPDUMP' && !'SDMPDUMP' && 
+    if((hdr.cPages > hdr.cBits) || (hdr.cBits > 0x7fffffff) || (hdr.cbFileBase & 0xfff) || (hdr.cbFileBase > 0x01000000)) { goto fail; }
+    cbFileBase = hdr.cbFileBase;
+    // 2: fetch bits:
+    cb = hdr.cBits / 8;
+    if(!(pb = LocalAlloc(LMEM_ZEROINIT, (SIZE_T)cb))) { goto fail; }
+    if(cb != fread(pb, 1, (SIZE_T)cb, ctx->pFile)) { goto fail; }
+    // 3: walk bitmap - add ranges!
+    cMaxBits = hdr.cBits & 0xffffffc0;
+    for(iPage = 0; iPage < cMaxBits; iPage += 64) {
+        b = *(PQWORD)(pb + (iPage >> 3));
+        // all pages valid!
+        if(b == (QWORD)-1) {
+            if(!fPageValid) {
+                fPageValid = TRUE;
+                iPageBase = iPage;
+            }
+            continue;
+        }
+        // no pages valid!
+        if(!b && !fPageValid) { continue; }
+        // some valid pages
+        for(iPageEx = 0; iPageEx < 64; iPageEx++) {
+            if((b >> iPageEx) & 1) {
+                // valid
+                if(!fPageValid) {
+                    fPageValid = TRUE;
+                    iPageBase = iPage + iPageEx;
+                }
+            } else {
+                // invalid
+                if(fPageValid) {
+                    cb = (iPage + iPageEx - iPageBase) << 12;
+                    if(!LcMemMap_AddRange(ctxLC, iPageBase << 12, cb, cbFileBase)) {
+                        lcprintf(ctxLC, "DEVICE: FAIL: unable to add range to memory map. (%016llx %016llx %016llx)\n", iPageBase << 12, cb, cbFileBase);
+                        return FALSE;
+                    }
+                    cbFileBase += cb;
+                    fPageValid = FALSE;
+                }
+            }
+        }
+    }
+    // 4: finalize remaining
+    if(fPageValid) {
+        cb = (iPage - iPageBase) << 12;
+        if(!LcMemMap_AddRange(ctxLC, iPageBase << 12, cb, cbFileBase)) {
+            lcprintf(ctxLC, "DEVICE: FAIL: unable to add range to memory map. (%016llx %016llx %016llx)\n", iPageBase << 12, cb, cbFileBase);
+            return FALSE;
+        }
+    }
+    fResult = TRUE;
+fail:
+    if(!fResult) {
+        lcprintf(ctxLC, "DEVICE: FAIL: error parsing 64-bit full bitmap dump.\n");
+    }
+    return fResult;
+}
+
 /*
 * Try to initialize a Microsoft Crash Dump file (full dump only) _or_ a VirtualBox
 * core dump file. This is done by reading the dump header. If this is not a
@@ -327,6 +412,13 @@ BOOL DeviceFile_MsCrashCoreDumpInitialize(_In_ PLC_CONTEXT ctxLC)
             cbFileOffset += pM64->Run[i].PageCount << 12;
         }
     }
+    if((CDMP_DWORD(0x000) == DUMP_SIGNATURE) && (CDMP_DWORD(0x004) == DUMP_VALID_DUMP64) && (CDMP_DWORD(0xf98) == DUMP_TYPE_BITMAP_FULL) && (CDMP_DWORD(0x030) == IMAGE_FILE_MACHINE_AMD64)) {
+        return DeviceFile_MsCrashCoreDumpInitialize_BitmapFullOrActiveMemory(ctxLC);
+    }
+    if((CDMP_DWORD(0x000) == DUMP_SIGNATURE) && (CDMP_DWORD(0x004) == DUMP_VALID_DUMP64) && (CDMP_DWORD(0xf98) == DUMP_TYPE_ACTIVE_MEMORY) && (CDMP_DWORD(0x030) == IMAGE_FILE_MACHINE_AMD64)) {
+        lcprintfv(ctxLC, "DEVICE: WARN: active only memory dump - analysis will be degraded!\n");
+        return DeviceFile_MsCrashCoreDumpInitialize_BitmapFullOrActiveMemory(ctxLC);
+    }
     if((CDMP_DWORD(0x000) == DUMP_SIGNATURE) && (CDMP_DWORD(0x004) == DUMP_VALID_DUMP) && (CDMP_DWORD(0xf88) == DUMP_TYPE_FULL) && (CDMP_DWORD(0x020) == IMAGE_FILE_MACHINE_I386)) {
         // PAGEDUMP (32-bit memory dump) and FULL DUMP
         lcprintfvv_fn(ctxLC, "32-bit Microsoft Crash Dump identified.\n");
@@ -517,6 +609,7 @@ BOOL DeviceFile_Open(_Inout_ PLC_CONTEXT ctxLC, _Out_opt_ PPLC_CONFIG_ERRORINFO
     if(_fseeki64(ctx->pFile, 0, SEEK_END)) { goto fail; }   // seek to end of file
     ctx->cbFile = _ftelli64(ctx->pFile);                    // get current file pointer
     if(ctx->cbFile < 0x01000000) { goto fail; }             // minimum allowed dump file size = 16MB
+    if(ctx->cbFile > 0xffff000000000000) { goto fail; }     // file too large
     ctxLC->hDevice = (HANDLE)ctx;
     // set callback functions and fix up config
     ctxLC->pfnClose = DeviceFile_Close;
